-- ===============================================================================
-- MODULE       DL710_LogicPool
-- PROJECT      LogicBox
-- AUTHOR       Esteban Rubio, PhysIHD
-- -------------------------------------------------------------------------------
-- ?                    Init
-- 16.4.10      V3.1
-- -------------------------------------------------------------------------------
-- Top entity for DL710 with LogicPool via CBus
-- ===============================================================================

library ieee;
use ieee.std_logic_1164.all;

use work.LogicBox_pkg.all;
use work.svn_extract.all;

entity DL710 is
--generic(
--    PRD_ID              : std_logic_vector(31 downto 0) := (others => '1'));
port (
    DL710_I_CLK100      : in     std_logic;
    -- Board LEDs
    DL710_O_LED_PWR     : out    std_logic;
    DL710_O_LED_SEL     : out    std_logic;
    DL710_O_LED_INT     : out    std_logic := '0';
    DL710_O_LED_MOD     : out    std_logic := '0';
    -- put here the signals to the SUxxx modules
    -- SU_BEGIN  <- please don't delete or modify this comment!
    -- SU_END
    -- end of the SUxxx section
    -- VME signals
    DL710_I_SYSRESET    : in     std_logic;
    DL710_I_NDS0        : in     std_logic;
    DL710_I_NDS1        : in     std_logic;
    DL710_I_NAS         : in     std_logic;
    DL710_I_NWRITE      : in     std_logic;
    DL710_I_NLWORD      : in     std_logic;
    DL710_I_AM          : in     std_logic_vector ( 5 downto  0);
    DL710_I_STD_ADR     : in     std_logic_vector (23 downto 19); -- Base address of the card
    DL710_I_IADR        : in     std_logic_vector (18 downto  1); -- Intern address of the card

    DL710_O_NDTACK      : out    std_logic;
    DL710_IO_IDAT       : inout  std_logic_vector (31 downto  0);

    DL710_O_NIRQ1       : out    std_logic;
    DL710_O_NIRQ2       : out    std_logic;
    DL710_O_NIRQ3       : out    std_logic;
    DL710_O_NIRQ4       : out    std_logic;
    DL710_O_NIRQ5       : out    std_logic;
    DL710_O_NIRQ6       : out    std_logic;
    DL710_O_NIRQ7       : out    std_logic;
    DL710_I_NIACKIN     : in     std_logic;
    DL710_O_NIACKOUT    : out    std_logic;
    -- Control to vmebuffers
    DL710_I_SWI_ADR     : in     std_logic_vector ( 7 downto 0); -- Base address of the switch
    DL710_O_NEN_BUFF    : inout  std_logic; -- enable the data buffer to read/write data
    DL710_O_NDAT2VME    : inout  std_logic);
end DL710;


--+-----------------------------------------------------------------------------+
--|                         ARCHITECTURE                                        |
--+-----------------------------------------------------------------------------+

architecture Behavioral of DL710 is

component VME2CBUS is
Generic (IDNr           : std_logic_vector(31 downto 0) := (others => '1');
         SVN_ID         : std_logic_vector(31 downto 0) := (others => '1');
--         PRD_ID     : std_logic_vector(31 downto 0) := (others => '1');
         CRDY_CYC       : natural := 1);
        Port
   (
      I_CLK100          : in    std_logic;
      -- VME signals
      I_SYSRESET        : in    std_logic;
      I_NDS0            : in    std_logic;
      I_NDS1            : in    std_logic;
      I_NAS             : in    std_logic;
      I_NWRITE          : in    std_logic;
      I_NLWORD          : in    std_logic;
      I_AM              : in    std_logic_vector ( 5 downto  0);
      I_STD_ADR         : in    std_logic_vector (23 downto 19); -- Base address of the card
      I_IADR            : in    std_logic_vector (18 downto  1); -- Intern address of the card

      O_NDTACK          : out   std_logic;
      IO_IDAT           : inout std_logic_vector (31 downto  0);

      O_NIRQ1           : out   std_logic;
      O_NIRQ2           : out   std_logic;
      O_NIRQ3           : out   std_logic;
      O_NIRQ4           : out   std_logic;
      O_NIRQ5           : out   std_logic;
      O_NIRQ6           : out   std_logic;
      O_NIRQ7           : out   std_logic;
      I_NIACKIN         : in    std_logic;
      O_NIACKOUT        : out   std_logic;

      -- Control to vmebuffers
      I_SWI_ADR         : in    std_logic_vector ( 7 downto  0); -- Base address of the switch
      O_NEN_BUFF        : inout std_logic; -- enable the data buffer to read/write data
      O_NDAT2VME        : inout std_logic;

      -- Board LEDs
      O_LED_PWR         : out   std_logic;
      O_LED_SEL         : out   std_logic;
      O_LED_INT         : out   std_logic := '0';
      O_LED_MOD         : out   std_logic := '0';

      -- CBUS interface
      O_Reset           : out   std_logic;
      -- IF
      O_CAddr           : out   std_logic_vector(31 downto  0);
      O_CWR             : out   std_logic;
      O_CRD             : out   std_logic;
      O_CDIn            : out   std_logic_vector(31 downto  0);
      I_CDOut           : in    std_logic_vector(31 downto  0);
      I_CRdy            : in    std_logic:= 'Z'
   );
end component;

component top_core is
Generic(ClkMHz  : integer := 100);
Port (
    CLK                 : in    std_logic;
    Reset               : in    std_logic;
    -- put here the signals to the SUxxx modules
    -- SU_BEGIN  <- please don't delete or modify this comment!
    -- SU_END
    -- end of the SUxxx section
    Tick1ms             : in    std_logic;
    Tick20ms            : in    std_logic;
    -- DMA config & handshaking
    DMA_ena             : in    std_logic;                     -- DMA enabled
    DMA_ack             : in    std_logic;                     -- DMA acknoledge
    DMA_req             : out   std_logic;                     -- request, activate only when DMA_ena=1, deactivate after ack
    DMA_Nwords          : out   std_logic_vector(15 downto 0); -- Number of words
    DMA_Addr            : out   std_logic_vector(31 downto 0); -- start address
    DMA_dsize           : out   std_logic_vector( 1 downto 0); -- 0..3 for 1..4 bytes
    DMA_Ainc            : out   std_logic;                     -- auto increment of the address
    -- IF
    CAddr               : in    std_logic_vector(31 downto  0);
    CWR                 : in    std_logic;
    CRD                 : in    std_logic;
    CDIn                : in    std_logic_vector(31 downto  0);
    CDOut               : out   std_logic_vector(31 downto  0);
    CRdy                : out   std_logic);
end component;

component Ticker is
    Generic(Divider     : integer := 100);
    Port (  CLK         : in  std_logic;
            -- Ticks
            Input       : in  std_logic := '1';
            Output      : out std_logic);
end component;

   -- LogicPool Interface
signal CLK              : std_logic;                       -- out
signal eruReset         : std_logic;                       -- out
signal Tick1ms          : std_logic;                       -- out
signal Tick20ms         : std_logic;                       -- out
-- IF
signal CAddr            : std_logic_vector(31 downto 0);   --  out
signal CWR              : std_logic;                       -- out
signal CRD              : std_logic;                       -- out
signal CDIn             : std_logic_vector(31 downto 0);   --  in
signal CDOut            : std_logic_vector(31 downto 0);   --  out
signal CRdy             : std_logic := '1';                --  in

-- not implemented
signal DMA_ena          : std_logic;                     -- DMA enabled
signal DMA_ack          : std_logic;                     -- DMA acknoledge
--signal DMA_req          : std_logic;                     -- request, activate only when DMA_ena=1, deactivate after ack
--signal DMA_Nwords       : std_logic_vector(15 downto 0); -- Number of words
--signal DMA_Addr         : std_logic_vector(31 downto 0); -- start address
--signal DMA_dsize        : std_logic_vector( 1 downto 0); -- 0..3 for 1..4 bytes
--signal DMA_Ainc         : std_logic;                     -- auto increment of the address

constant svn_rev        : string := "$Rev: 361 $";
constant svn_date       : string := "$Date: 2013-07-26 14:35:41 +0200 (Fri, 26 Jul 2013) $";
constant svn_id         : std_logic_vector(31 downto 0) := DateSVN(revision => svn_rev, datestr => svn_date);

begin
    DMA_ena <= '0';
    DMA_ack <= '0';

    CLK <= DL710_I_CLK100;

Ticker1ms : Ticker
    Generic map(Divider =>  ClkMhz * 1000)
    Port map(   CLK     => CLK,
                -- Ticks
                Input   => '1',
                Output  => Tick1ms);

Ticker20ms : Ticker
    Generic map(Divider =>  10)
    Port map(   CLK     => CLK,
                -- Ticks
                Input   => Tick1ms,
                Output  => Tick20ms);

Vme2CBus_cntrl: VME2CBUS
Generic map(
    IDNr                => DateTimeVID,
    SVN_ID              => svn_id)
Port map
(
    I_CLK100        => CLK,
    -- VME signals
    I_SYSRESET      => DL710_I_SYSRESET,
    I_NDS0          => DL710_I_NDS0,
    I_NDS1          => DL710_I_NDS1,
    I_NAS           => DL710_I_NAS,
    I_NWRITE        => DL710_I_NWRITE,
    I_NLWORD        => DL710_I_NLWORD,
    I_AM            => DL710_I_AM,
    I_STD_ADR       => DL710_I_STD_ADR,
    I_IADR          => DL710_I_IADR,

    O_NDTACK        => DL710_O_NDTACK,
    IO_IDAT         => DL710_IO_IDAT,

    O_NIRQ1         => DL710_O_NIRQ1,
    O_NIRQ2         => DL710_O_NIRQ2,
    O_NIRQ3         => DL710_O_NIRQ3,
    O_NIRQ4         => DL710_O_NIRQ4,
    O_NIRQ5         => DL710_O_NIRQ5,
    O_NIRQ6         => DL710_O_NIRQ6,
    O_NIRQ7         => DL710_O_NIRQ7,
    I_NIACKIN       => DL710_I_NIACKIN,
    O_NIACKOUT      => DL710_O_NIACKOUT,

    -- Control to vmebuffers
    I_SWI_ADR      => DL710_I_SWI_ADR,
    O_NEN_BUFF     => DL710_O_NEN_BUFF,
    O_NDAT2VME     => DL710_O_NDAT2VME,

    -- Board LEDs
    O_LED_PWR      => DL710_O_LED_PWR,
    O_LED_SEL      => DL710_O_LED_SEL,
    O_LED_INT      => DL710_O_LED_INT,
    O_LED_MOD      => DL710_O_LED_MOD,

    -- CBUS interface
    O_Reset        => eruReset,
    -- IF
    O_CAddr        => CAddr,
    O_CWR          => CWR,
    O_CRD          => CRD,
    O_CDIn         => CDIn,
    I_CDOut        => CDOut,
    I_CRdy         => CRdy
);

LP: top_core
 Generic Map(
    ClkMHz              => ClkMHz)
 Port Map(
    CLK                 => CLK,
    Reset               => eruReset,
    -- put here the mapping of the signals to the SUxxx modules
    -- SU_MAP_BEGIN  <- please don't delete or modify this comment!
    -- SU_MAP_END
    -- end of the SUxxx section
    Tick1ms             => Tick1ms,
    Tick20ms            => Tick20ms,
    -- DMA config & handshaking - not implemented!
    DMA_ena             => DMA_ena,
    DMA_ack             => DMA_ack,
    DMA_req             => open,
    DMA_Nwords          => open,
    DMA_Addr            => open,
    DMA_dsize           => open,
    DMA_Ainc            => open,
    -- IF
    CAddr               => CAddr,
    CWR                 => CWR,
    CRD                 => CRD,
    CDIn                => CDIn,
    CDOut               => CDOut,
    CRdy                => CRdy);

end;
